// c0   - (0.125, 0.5, 1.0, 2.0)
// c2   - World Space Camera Position
// c4-c7- World/View/Projection Matrix
// c8-c11 - World Matrix

// v0   - Vertex Position
// v3   - Vertex Normal
// v7   - Vertex Texture Data u,v 
// v8   - Vertex tangent Vector
vs.1.1

dcl_position v0
dcl_normal v3
dcl_texcoord v7
dcl_tangent v8

m4x4  oPos, v0, c4              // OutPos = ObjSpacePos * WVP Matrix
mov   oT0, v7                   // output basemap texcoords


;The next section of the vertex shader computes the view vector in world space. 
;This is accomplished by subtracting the world space camera position from the world
;space vertex position and then normalizing the result.  Note that in this example 
;the world transform is identity and the input vertex position is used directly.  
;If this is not the case in your application, you will need to transform the input
;position by the world matrix before performing this calculation.


m4x4    r10, v0, c8       // transform vertex position

sub   r8, c2, r10                // viewer direction
dp3   r10.x, r8, r8             // magnitude
rsq   r10.y, r10.x              // normalize
mul   r8, r8, r10.y             // V normalized view vector
mov   oT1, r8                   // world space view vector


;The next section passes along the tangent/texture space basis matrix. In this
;example we compute the binormal by taking the cross product of the tangent and 
;the normal. Note that this is not a general solution it relies upon the fact 
;that the texture coordinates for this model are not reversed on any triangles.  


m3x3   r5, v3, c8                    // normal (tZ)
mov   oT4, r5

m3x3   r3, v8, c8                    // tangent (tY)
mov   oT2, r3
mul   r4, r5.yzxw, r3.zxyw      // compute binormal (tX) as being 
mad   r4, r5.zxyw, -r3.yzxw, r4 // perpendicular to normal
mov   oT3, r4

